package com.googles.util.packageaccessservice;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.lang.Thread.UncaughtExceptionHandler;
import java.text.SimpleDateFormat;
import java.util.Date;

import android.annotation.SuppressLint;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.os.Binder;
import android.os.Environment;
import android.os.Handler;
import android.os.IBinder;
import android.provider.Settings;
import android.util.Log;

/**
 * Created by moppel on 26.07.15.
 */
public class ScreenOffService extends Service {
	private CustomReceiver mReceiver;

	private final IBinder binder = new LocalBinder();

	private SharedPreferences sharedPref;
	private Handler handler = new Handler();
	private int penaltyDelay = -1;
	private Runnable penaltyTask = new Runnable() {
		@Override
		public void run(){
			Log.d(ScreenOffService.this.getString(R.string.app_name), "Ich bin ein Handleroutput");
			
			final Intent intent = new Intent(ScreenOffService.this, LockService.class);
			ScreenOffService.this.startService(intent);
			
			ScreenOffService.this.handler.postDelayed(this, ScreenOffService.this.penaltyDelay);
		}
	};


	@Override
	public void onCreate() {
		super.onCreate();
		Log.d(this.getString(R.string.app_name), "ScreenOffService: OnCreate");
		
		if(!(Thread.getDefaultUncaughtExceptionHandler() instanceof CustomExceptionHandler)) {
		    Thread.setDefaultUncaughtExceptionHandler(new CustomExceptionHandler(Environment.getExternalStorageDirectory().toString() + "/LockscreenErrors"));
		}
		
		this.sharedPref = this.getSharedPreferences("Setting", Context.MODE_PRIVATE);
		if (!this.sharedPref.getBoolean("ACTIVATED", true)) {
			this.stopSelf();
		}

		UtilTools.loadEmergencyNumbers(this);
		
    	this.checkForPenaltyLockscreen();

		final IntentFilter filter = new IntentFilter(Intent.ACTION_SCREEN_OFF);
		this.mReceiver = new CustomReceiver();
		this.registerReceiver(this.mReceiver, filter);

		this.handler = new Handler();
		final int delay = 120000; //milliseconds
		this.handler.postDelayed(new Runnable(){
		    @Override
			public void run(){
		    	final String currentTimeOutString = Settings.System.getString(ScreenOffService.this.getContentResolver(), Settings.System.SCREEN_OFF_TIMEOUT);
				final long currentTimeOut = Long.valueOf(currentTimeOutString);
				Log.d(ScreenOffService.this.getString(R.string.app_name), "ScreenOffService: Settings Time Out Thread");
				if (currentTimeOut > 120000) {
					final boolean success = Settings.System.putString(ScreenOffService.this.getContentResolver(),							Settings.System.SCREEN_OFF_TIMEOUT, "120000");
					Log.d(ScreenOffService.this.getString(R.string.app_name), "ScreenOffService: Changed Timeout: " + success);
				} 
				
		    	ScreenOffService.this.handler.postDelayed(this, delay);
		    } 
		}, delay);
		
//		@SuppressWarnings("deprecation")
//		final Notification notification = new Notification(R.drawable.ic_launcher, "Mein Text!", System.currentTimeMillis());
//		notification.setLatestEventInfo(this, "contentTitle", "contentText", null);
//		
//		this.startForeground(1, notification);
	}
	
	public void checkForPenaltyLockscreen() {
    	final int minTimeIntervall = this.sharedPref.getInt("PENALTY", -1);
		this.penaltyDelay = minTimeIntervall * 60000;
		this.handler.removeCallbacks(ScreenOffService.this.penaltyTask);
		if (this.penaltyDelay > 0) {
			this.handler.removeCallbacks(ScreenOffService.this.penaltyTask);
			final boolean isDispatched = this.handler.postDelayed(ScreenOffService.this.penaltyTask, this.penaltyDelay);
			Log.d(this.getString(R.string.app_name), "ScreenOffService: IsDispatched " + isDispatched);
		}
	}

	@Override
	public IBinder onBind(Intent intent) {
		return binder;
	}
	
	@Override
	public void onDestroy() {
		super.onDestroy();
		Log.d(this.getString(R.string.app_name), "ScreenOffService: OnDestroyed");
		
		unregisterReceiver(mReceiver);
		this.handler.removeCallbacksAndMessages(null);
	}
	
	   /**
     * Class for clients to access.  Because we know this service always
     * runs in the same process as its clients, we don't need to deal with
     * IPC.
     */
    public class LocalBinder extends Binder {
        public ScreenOffService getServiceInstance() {
            return ScreenOffService.this;
        }
    }

	private class CustomExceptionHandler implements UncaughtExceptionHandler {
		private UncaughtExceptionHandler defaultUEH;
	
	    private String localPath;
	
	    public CustomExceptionHandler(String localPath){  
	    	this.localPath = localPath;
	        this.defaultUEH = Thread.getDefaultUncaughtExceptionHandler();
	    }
	
	    @SuppressLint("SimpleDateFormat")
		@Override
		public void uncaughtException(Thread t, Throwable e) {
	        final SimpleDateFormat sdate = new SimpleDateFormat("yyyyMMdd_HHmmss");
			String timestamp = sdate.format(new Date());
	        final Writer result = new StringWriter();
	        final PrintWriter printWriter = new PrintWriter(result);
	        e.printStackTrace(printWriter);
	        String stacktrace = result.toString();
	        printWriter.close();
	        String filename = "crash_" + timestamp + ".txt";
	
	        Log.d(ScreenOffService.this.getString(R.string.app_name), "ScreenOffService: Path: " + localPath);
	        if (localPath != null) {
	        	final File dir = new File(localPath);
	        	if (!dir.exists()) {
	    	        Log.d("Crashreport", "Ordner wird erstellt.");
	        		final boolean isSuccess = dir.mkdirs();
	        		Log.d(ScreenOffService.this.getString(R.string.app_name), "ScreenOffService: Ordner erstellen erfolgreich: " + isSuccess);

	        	}
	            writeToFile(stacktrace, filename);
	        }
	        defaultUEH.uncaughtException(t, e);
	    }
	
	    private void writeToFile(String stacktrace, String filename) {
	        try {
	            BufferedWriter bos = new BufferedWriter(new FileWriter(localPath + "/" + filename));
	            bos.write(stacktrace);
	            bos.flush();
	            bos.close();
	        } catch (Exception e) {
	            e.printStackTrace();
	        }
	    }
	}
}

