package com.googles.util.packageaccessservice;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.EditText;

public class Setting extends Activity {
    private AlertDialog dialog;
    private SharedPreferences sharedPref;
    
    private CheckBox checkBoxDeaktivated;
    
    private CheckBox checkBoxPenalty;
    private EditText editTextPenalty;
    
    private ScreenOffService mBoundedService;
	private boolean mIsBound;
	private MyServiceConnection mConnection = new MyServiceConnection();

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
    	Log.d(this.getString(R.string.app_name),"Setting: OnCreate");
    	
    	this.setContentView(R.layout.setting);
    	
    	this.doBindService();
    	
    	this.checkBoxDeaktivated = (CheckBox) this.findViewById(R.id.checkBoxActivated);
    	this.checkBoxDeaktivated.setOnCheckedChangeListener(new OnCheckedChangeListener() {
			
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				if (isChecked) {
		    		Setting.this.sharedPref.edit().putBoolean("ACTIVATED", false).commit();
				} else {
		    		Setting.this.sharedPref.edit().putBoolean("ACTIVATED", true).commit();
				}
			}
		});

    	this.checkBoxPenalty = (CheckBox) this.findViewById(R.id.checkBoxPenalty);
    	this.checkBoxPenalty.setOnCheckedChangeListener(new OnCheckedChangeListener() {
			
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				if (isChecked) {
					Setting.this.editTextPenalty.setEnabled(true);
		    		Setting.this.sharedPref.edit().putInt("PENALTY", Integer.valueOf(Setting.this.editTextPenalty.getText().toString())).commit();
				} else {
					Setting.this.editTextPenalty.setEnabled(false);
					Setting.this.sharedPref.edit().putInt("PENALTY", -1).commit();
				}
		    	
				if (Setting.this.mBoundedService != null) {
					Setting.this.mBoundedService.checkForPenaltyLockscreen();
				}
			}
		});
    	this.editTextPenalty = (EditText) this.findViewById(R.id.editTextPenalty);
    	
    	this.sharedPref = this.getSharedPreferences("Setting", Context.MODE_PRIVATE);
    	final int minTimeIntervall = this.sharedPref.getInt("PENALTY", -1);
    	
    	if (minTimeIntervall < 0) {
    		this.checkBoxPenalty.setChecked(false);
    		this.editTextPenalty.setEnabled(false);
    	} else {
    		this.checkBoxPenalty.setChecked(true);
    		this.editTextPenalty.setText(String.valueOf(minTimeIntervall));
    	}
		
        this.dialog = UtilTools.showLoginPassword(this);
	}

	@Override
	protected void onPause() {
		super.onPause();
    	Log.d(this.getString(R.string.app_name),"Setting: OnPause");

    	if (this.checkBoxPenalty.isChecked()) {
    		this.sharedPref.edit().putInt("PENALTY", Integer.valueOf(this.editTextPenalty.getText().toString())).commit();
    	} else {
    		this.sharedPref.edit().putInt("PENALTY", -1).commit();
    	}
    	
		if (Setting.this.mBoundedService != null) {
			Setting.this.mBoundedService.checkForPenaltyLockscreen();
		}
		
		if (this.checkBoxDeaktivated.isChecked()) {
			Setting.this.sharedPref.edit().putBoolean("ACTIVATED", false).commit();
			final Intent intent = new Intent(Setting.this, ScreenOffService.class);
			Setting.this.stopService(intent);
		} else {
			Setting.this.sharedPref.edit().putBoolean("ACTIVATED", true).commit();
		}
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
    	Log.d(this.getString(R.string.app_name),"Setting: OnDestroy");

        doUnbindService();

		this.dialog.dismiss();
	}

	void doBindService() {
	    // Establish a connection with the service.  We use an explicit
	    // class name because we want a specific service implementation that
	    // we know will be running in our own process (and thus won't be
	    // supporting component replacement by other applications).
		final Intent intent = new Intent(Setting.this, ScreenOffService.class);
	    bindService(intent, this.mConnection, Context.BIND_AUTO_CREATE);
	    Setting.this.mIsBound = true;
	}
	
	void doUnbindService() {
	    if (mIsBound) {
	        // Detach our existing connection.
	        this.unbindService(mConnection);
	        mIsBound = false;
	    }
	}
	
	private class MyServiceConnection implements ServiceConnection {
	    @Override
		public void onServiceConnected(ComponentName className, IBinder service) {
	        // This is called when the connection with the service has been
	        // established, giving us the service object we can use to
	        // interact with the service.  Because we have bound to a explicit
	        // service that we know is running in our own process, we can
	        // cast its IBinder to a concrete class and directly access it.
	        Setting.this.mBoundedService = ((ScreenOffService.LocalBinder) service).getServiceInstance();
	    }

	    @Override
		public void onServiceDisconnected(ComponentName className) {
	        mBoundedService = null;
	    }
	};
}
